import logging
from contextlib import asynccontextmanager
from typing import AsyncGenerator

from fastapi import FastAPI

from fastfood_two.application.common.logger import configure_logger
from fastfood_two.presentation.fastapi_backend.dependencies import init_dependencies
from fastfood_two.presentation.fastapi_backend.error_handlers import init_errorhandlers
from fastfood_two.presentation.fastapi_backend.routers_init import init_routers

logger = logging.getLogger(__name__)
configure_logger(level=logging.INFO)


@asynccontextmanager
async def app_lifespan(app: FastAPI) -> AsyncGenerator:
    """Application lifespan.

    This function is called when the application starts and stops.
    Do not use this method directly, use the FastAPI `lifespan` instead.

    :param app: FastAPI application
    :type app: FastAPI
    """

    logger.info("Application lifespan started")

    yield

    logger.info("Application lifespan stopped")


def app_factory() -> FastAPI:
    """Create a FastAPI application.

    Create a FastAPI application and initialize all dependencies.

    :return: FastAPI
    :rtype: FastAPI
    """

    app = FastAPI(
        lifespan=app_lifespan,
    )

    init_dependencies(app)
    init_errorhandlers(app)
    init_routers(app)

    return app
